@@ -8,7 +8,6 @@ possibly, convert the result into a more managable format.
 The following packages are used:
 
   - for JSON, the U{simplejson<http://cheeseshop.python.org/pypi/simplejson>} package: C{http://cheeseshop.python.org/pypi/simplejson}
-  - for XML, the U{PyXML<http://pyxml.sourceforge.net/>}: C{http://pyxml.sourceforge.net/}
   - for RDF/XML, the U{RDFLib<http://rdflib.net>}: C{http://rdflib.net}
 
 These packages are imported in a lazy fashion, ie, only when needed. Ie, if the user never intends to use the
@@ -50,7 +49,7 @@ Automatic conversion of the results
 
 To make processing somewhat easier, the package can do some conversions automatically from the return result. These are:
 
-  - for XML, the U{PyXML<http://pyxml.sourceforge.net/>} (C{http://pyxml.sourceforge.net/}) is
+  - for XML, the U{xml.dom.minidom<http://docs.python.org/library/xml.dom.minidom.html>} (C{http://docs.python.org/library/xml.dom.minidom.html}) is
   used to convert the result stream into a Python representation of a DOM tree
   - for JSON, the U{simplejson<http://cheeseshop.python.org/pypi/simplejson>} package (C{http://cheeseshop.python.org/pypi/simplejson}) to generate a Python dictionary
 
@@ -159,18 +158,16 @@ The package was greatly inspired by U{Lee Feigenbaum's similar package for Javas
 @organization: U{World Wide Web Consortium<http://www.w3.org>} and U{Foundation CTIC<http://www.fundacionctic.org/>}.
 @license: U{W3C® SOFTWARE NOTICE AND LICENSE<href="http://www.w3.org/Consortium/Legal/copyright-software">}
 @requires: U{simplejson<http://cheeseshop.python.org/pypi/simplejson>} package.
-@requires: U{PyXML<http://pyxml.sourceforge.net/>} package.
 @requires: U{RDFLib<http://rdflib.net>} package.
 """
 
-__version__ = "1.2.0"
+__version__ = "1.4.1"
 __authors__  = u"Ivan Herman, Sergio Fernández, Carlos Tejo Alonso"
 __license__ = u'W3C® SOFTWARE NOTICE AND LICENSE, http://www.w3.org/Consortium/Legal/copyright-software'
 __contact__ = 'sparql-wrapper-devel@lists.sourceforge.net'
-__date__    = "2008-07-10"
+__date__    = "2010-02-12"
 __agent__   = "sparqlwrapper %s (http://sparql-wrapper.sourceforge.net/)" % __version__
 
-from Wrapper      import SPARQLWrapper, XML, JSON, TURTLE, N3, GET, POST, SELECT, CONSTRUCT, ASK, DESCRIBE
+from Wrapper      import SPARQLWrapper, XML, JSON, TURTLE, N3, RDF, GET, POST, SELECT, CONSTRUCT, ASK, DESCRIBE
 from SmartWrapper import SPARQLWrapper2
 
-
